@@ -15,9 +15,22 @@ import click
 
 from . import watson as _watson
 from .frames import Frame
-from .utils import (apply_weekday_offset, format_timedelta,
-                    get_frame_from_argument, get_start_time_for_period,
-                    options, safe_save, sorted_groupby, style, parse_tags)
+from .utils import (
+    apply_weekday_offset,
+    build_csv,
+    confirm_project,
+    confirm_tags,
+    flatten_report_for_csv,
+    format_timedelta,
+    frames_to_csv,
+    frames_to_json,
+    get_frame_from_argument,
+    get_start_time_for_period,
+    options, safe_save,
+    sorted_groupby,
+    style,
+    parse_tags,
+)
 
 
 class MutuallyExclusiveOption(click.Option):
@@ -26,19 +39,26 @@ class MutuallyExclusiveOption(click.Option):
         super(MutuallyExclusiveOption, self).__init__(*args, **kwargs)
 
     def handle_parse_result(self, ctx, opts, args):
-        if self.mutually_exclusive.intersection(opts) and self.name in opts:
-            raise click.UsageError(
-                '`--{name}` is mutually exclusive with the following options: '
-                '{options}'.format(name=self.name.replace('_', ''),
-                                   options=', '
-                                   .join(['`--{}`'.format(_) for _ in
-                                         self.mutually_exclusive]))
-            )
-
+        if self.name in opts:
+            if self.mutually_exclusive.intersection(opts):
+                self._raise_exclusive_error()
+            if self.multiple and len(set(opts[self.name])) > 1:
+                self._raise_exclusive_error()
         return super(MutuallyExclusiveOption, self).handle_parse_result(
             ctx, opts, args
         )
 
+    def _raise_exclusive_error(self):
+        # Use self.opts[-1] instead of self.name to handle options with a
+        # different internal name.
+        self.mutually_exclusive.add(self.opts[-1].strip('-'))
+        raise click.UsageError(
+            'The following options are mutually exclusive: '
+            '{options}'.format(options=', '
+                               .join(['`--{}`'.format(_) for _ in
+                                     self.mutually_exclusive]))
+        )
+
 
 class WatsonCliError(click.ClickException):
     def format_message(self):
@@ -53,12 +73,14 @@ class DateParamType(click.ParamType):
 
     def convert(self, value, param, ctx):
         if value:
-            date = arrow.get(value)
+            try:
+                date = arrow.get(value)
+            except arrow.parser.ParserError as e:
+                raise click.UsageError(str(e))
             # When we parse a date, we want to parse it in the timezone
             # expected by the user, so that midnight is midnight in the local
             # timezone, not in UTC. Cf issue #16.
             date.tzinfo = tz.tzlocal()
-
             # Add an offset to match the week beginning specified in the
             # configuration
             if param.name == "week":
@@ -152,9 +174,13 @@ def _start(watson, project, tags, restart=False, gap=True):
               help=("(Don't) leave gap between end time of previous project "
                     "and start time of the current."))
 @click.argument('args', nargs=-1)
+@click.option('-c', '--confirm-new-project', is_flag=True, default=False,
+              help="Confirm addition of new project.")
+@click.option('-b', '--confirm-new-tag', is_flag=True, default=False,
+              help="Confirm creation of new tag.")
 @click.pass_obj
 @click.pass_context
-def start(ctx, watson, args, gap_=True):
+def start(ctx, watson, confirm_new_project, confirm_new_tag, args, gap_=True):
     """
     Start monitoring time for the given project.
     You can add tags indicating more specifically what you are working on with
@@ -177,9 +203,19 @@ def start(ctx, watson, args, gap_=True):
         itertools.takewhile(lambda s: not s.startswith('+'), args)
     )
 
+    # Confirm creation of new project if that option is set
+    if (watson.config.getboolean('options', 'confirm_new_project') or
+            confirm_new_project):
+        confirm_project(project, watson.projects)
+
     # Parse all the tags
     tags = parse_tags(args)
 
+    # Confirm creation of new tag(s) if that option is set
+    if (watson.config.getboolean('options', 'confirm_new_tag') or
+            confirm_new_tag):
+        confirm_tags(tags, watson.tags)
+
     if project and watson.is_started and not gap_:
         current = watson.current
         errmsg = ("Project {} is already started and '--no-gap' is passed. "
@@ -369,7 +405,7 @@ _SHORTCUT_OPTIONS_VALUES = {
 @click.option('-c/-C', '--current/--no-current', 'current', default=None,
               help="(Don't) include currently running frame in report.")
 @click.option('-f', '--from', 'from_', cls=MutuallyExclusiveOption, type=Date,
-              default=arrow.now().replace(days=-7),
+              default=arrow.now().shift(days=-7),
               mutually_exclusive=_SHORTCUT_OPTIONS,
               help="The date from when the report should start. Defaults "
               "to seven days ago.")
@@ -409,14 +445,32 @@ _SHORTCUT_OPTIONS_VALUES = {
               help="Reports activity only for frames containing the given "
               "tag. You can add several tags by using this option multiple "
               "times")
-@click.option('-j', '--json', 'format_json', cls=MutuallyExclusiveOption,
-              is_flag=True,
-              help="Format the report in JSON instead of plain text")
+@click.option('--ignore-project', 'ignore_projects', multiple=True,
+              help="Reports activity for all projects but the given ones. You "
+              "can ignore several projects by using the option multiple "
+              "times. Any given project will be ignored")
+@click.option('--ignore-tag', 'ignore_tags', multiple=True,
+              help="Reports activity for all tags but the given ones. You can "
+              "ignore several tags by using the option multiple times. Any "
+              "given tag will be ignored")
+@click.option('-j', '--json', 'output_format', cls=MutuallyExclusiveOption,
+              flag_value='json', mutually_exclusive=['csv'],
+              multiple=True,
+              help="Format output in JSON instead of plain text")
+@click.option('-s', '--csv', 'output_format', cls=MutuallyExclusiveOption,
+              flag_value='csv', mutually_exclusive=['json'],
+              multiple=True,
+              help="Format output in CSV instead of plain text")
+@click.option('--plain', 'output_format', cls=MutuallyExclusiveOption,
+              flag_value='plain', mutually_exclusive=['json', 'csv'],
+              multiple=True, default=True, hidden=True,
+              help="Format output in plain text (default)")
 @click.option('-g/-G', '--pager/--no-pager', 'pager', default=None,
               help="(Don't) view output through a pager.")
 @click.pass_obj
-def report(watson, current, from_, to, projects, tags, year, month,
-           week, day, luna, all, format_json, pager, aggregated=False):
+def report(watson, current, from_, to, projects, tags, ignore_projects,
+           ignore_tags, year, month, week, day, luna, all, output_format,
+           pager, aggregated=False):
     """
     Display a report of the time spent on each project.
 
@@ -434,15 +488,17 @@ def report(watson, current, from_, to, projects, tags, year, month,
     The shortcut `--luna` sets the timespan to the current moon cycle with
     the last full moon marking the start of the cycle.
 
-    You can limit the report to a project or a tag using the `--project` and
-    `--tag` options. They can be specified several times each to add multiple
-    projects or tags to the report.
+    You can limit the report to a project or a tag using the `--project`,
+    `--tag`, `--ignore-project` and `--ignore-tag` options. They can be
+    specified several times each to add or ignore multiple projects or
+    tags to the report.
 
     If you are outputting to the terminal, you can selectively enable a pager
     through the `--pager` option.
 
     You can change the output format for the report from *plain text* to *JSON*
-    by using the `--json` option.
+    using the `--json` option or to *CSV* using the `--csv` option. Only one
+    of these two options can be used at once.
 
     Example:
 
@@ -510,6 +566,15 @@ def report(watson, current, from_, to, projects, tags, year, month,
             "to": "2016-02-28T23:59:59.999999-08:00"
         }
     }
+    \b
+    $ watson report --from 2014-04-01 --to 2014-04-30 --project apollo11 --csv
+    from,to,project,tag,time
+    2014-04-01 00:00:00,2014-04-30 23:59:59,apollo11,,48140.0
+    2014-04-01 00:00:00,2014-04-30 23:59:59,apollo11,brakes,28421.0
+    2014-04-01 00:00:00,2014-04-30 23:59:59,apollo11,module,27701.0
+    2014-04-01 00:00:00,2014-04-30 23:59:59,apollo11,reactor,30950.0
+    2014-04-01 00:00:00,2014-04-30 23:59:59,apollo11,steering,38017.0
+    2014-04-01 00:00:00,2014-04-30 23:59:59,apollo11,wheels,36695.0
     """
 
     # if the report is an aggregate report, add whitespace using this
@@ -521,15 +586,19 @@ def report(watson, current, from_, to, projects, tags, year, month,
 
     try:
         report = watson.report(from_, to, current, projects, tags,
+                               ignore_projects, ignore_tags,
                                year=year, month=month, week=week, day=day,
                                luna=luna, all=all)
-    except watson.WatsonError as e:
-        raise click.ClickException(e)
+    except _watson.WatsonError as e:
+        raise click.ClickException(e.message)
 
-    if format_json and not aggregated:
+    if 'json' in output_format and not aggregated:
         click.echo(json.dumps(report, indent=4, sort_keys=True))
         return
-    elif format_json and aggregated:
+    elif 'csv' in output_format and not aggregated:
+        click.echo(build_csv(flatten_report_for_csv(report)))
+        return
+    elif 'plain' not in output_format and aggregated:
         return report
 
     lines = []
@@ -625,7 +694,7 @@ def report(watson, current, from_, to, projects, tags, year, month,
 @click.option('-c/-C', '--current/--no-current', 'current', default=None,
               help="(Don't) include currently running frame in report.")
 @click.option('-f', '--from', 'from_', cls=MutuallyExclusiveOption, type=Date,
-              default=arrow.now().replace(days=-7),
+              default=arrow.now().shift(days=-7),
               mutually_exclusive=_SHORTCUT_OPTIONS,
               help="The date from when the report should start. Defaults "
               "to seven days ago.")
@@ -641,15 +710,24 @@ def report(watson, current, from_, to, projects, tags, year, month,
               help="Reports activity only for frames containing the given "
               "tag. You can add several tags by using this option multiple "
               "times")
-@click.option('-j', '--json', 'format_json', cls=MutuallyExclusiveOption,
-              is_flag=True,
-              help="Format the report in JSON instead of plain text")
+@click.option('-j', '--json', 'output_format', cls=MutuallyExclusiveOption,
+              flag_value='json', mutually_exclusive=['csv'],
+              multiple=True,
+              help="Format output in JSON instead of plain text")
+@click.option('-s', '--csv', 'output_format', cls=MutuallyExclusiveOption,
+              flag_value='csv', mutually_exclusive=['json'],
+              multiple=True,
+              help="Format output in CSV instead of plain text")
+@click.option('--plain', 'output_format', cls=MutuallyExclusiveOption,
+              flag_value='plain', mutually_exclusive=['json', 'csv'],
+              multiple=True, default=True, hidden=True,
+              help="Format output in plain text (default)")
 @click.option('-g/-G', '--pager/--no-pager', 'pager', default=None,
               help="(Don't) view output through a pager.")
 @click.pass_obj
 @click.pass_context
-def aggregate(ctx, watson, current, from_, to, projects, tags,
-              format_json, pager):
+def aggregate(ctx, watson, current, from_, to, projects, tags, output_format,
+              pager):
     """
     Display a report of the time spent on each project aggregated by day.
 
@@ -667,8 +745,10 @@ def aggregate(ctx, watson, current, from_, to, projects, tags,
     If you are outputting to the terminal, you can selectively enable a pager
     through the `--pager` option.
 
-    You can change the output format for the report from *plain text* to *JSON*
-    by using the `--json` option.
+    You can change the output format from *plain text* to *JSON* using the
+    `--json` option or to *CSV* using the `--csv` option. Only one  of these
+    two options can be used at once.
+
 
     Example:
 
@@ -701,6 +781,21 @@ def aggregate(ctx, watson, current, from_, to, projects, tags,
     Wed 21 November 2018 - 01m 17s
       watson - 01m 17s
             [docs     01m 17s]
+    \b
+    $ watson aggregate --csv
+    from,to,project,tag,time
+    2018-11-14 00:00:00,2018-11-14 23:59:59,watson,,20542.0
+    2018-11-14 00:00:00,2018-11-14 23:59:59,watson,features,2046.0
+    2018-11-14 00:00:00,2018-11-14 23:59:59,watson,docs,18496.0
+    2018-11-19 00:00:00,2018-11-19 23:59:59,watson,,21532.0
+    2018-11-19 00:00:00,2018-11-19 23:59:59,watson,features,4323.0
+    2018-11-19 00:00:00,2018-11-19 23:59:59,watson,docs,17209.0
+    2018-11-20 00:00:00,2018-11-20 23:59:59,watson,,10235.0
+    2018-11-20 00:00:00,2018-11-20 23:59:59,watson,features,917.0
+    2018-11-20 00:00:00,2018-11-20 23:59:59,watson,docs,5863.0
+    2018-11-20 00:00:00,2018-11-20 23:59:59,watson,website,3455.0
+    2018-11-21 00:00:00,2018-11-21 23:59:59,watson,,77.0
+    2018-11-21 00:00:00,2018-11-21 23:59:59,watson,docs,77.0
     """
     delta = (to - from_).days
     lines = []
@@ -710,11 +805,13 @@ def aggregate(ctx, watson, current, from_, to, projects, tags,
         from_offset = from_ + offset
         output = ctx.invoke(report, current=current, from_=from_offset,
                             to=from_offset, projects=projects, tags=tags,
-                            format_json=format_json, pager=pager,
-                            aggregated=True)
+                            output_format=output_format,
+                            pager=pager, aggregated=True)
 
-        if format_json:
+        if 'json' in output_format:
             lines.append(output)
+        elif 'csv' in output_format:
+            lines.extend(flatten_report_for_csv(output))
         else:
             # if there is no activity for the day, append a newline
             # this ensures even spacing throughout the report
@@ -723,8 +820,10 @@ def aggregate(ctx, watson, current, from_, to, projects, tags,
 
             lines.append(u'\n'.join(output))
 
-    if format_json:
+    if 'json' in output_format:
         click.echo(json.dumps(lines, indent=4, sort_keys=True))
+    elif 'csv' in output_format:
+        click.echo(build_csv(lines))
     elif pager or (pager is None and
                    watson.config.getboolean('options', 'pager', True)):
         click.echo_via_pager(u'\n\n'.join(lines))
@@ -736,7 +835,7 @@ def aggregate(ctx, watson, current, from_, to, projects, tags,
 @click.option('-c/-C', '--current/--no-current', 'current', default=None,
               help="(Don't) include currently running frame in output.")
 @click.option('-f', '--from', 'from_', type=Date,
-              default=arrow.now().replace(days=-7),
+              default=arrow.now().shift(days=-7),
               help="The date from when the log should start. Defaults "
               "to seven days ago.")
 @click.option('-t', '--to', type=Date, default=arrow.now(),
@@ -773,13 +872,23 @@ def aggregate(ctx, watson, current, from_, to, projects, tags,
               help="Logs activity only for frames containing the given "
               "tag. You can add several tags by using this option multiple "
               "times")
-@click.option('-j', '--json', 'format_json', is_flag=True,
-              help="Format the log in JSON instead of plain text")
+@click.option('-j', '--json', 'output_format', cls=MutuallyExclusiveOption,
+              flag_value='json', mutually_exclusive=['csv'],
+              multiple=True,
+              help="Format output in JSON instead of plain text")
+@click.option('-s', '--csv', 'output_format', cls=MutuallyExclusiveOption,
+              flag_value='csv', mutually_exclusive=['json'],
+              multiple=True,
+              help="Format output in CSV instead of plain text")
+@click.option('--plain', 'output_format', cls=MutuallyExclusiveOption,
+              flag_value='plain', mutually_exclusive=['json', 'csv'],
+              multiple=True, default=True, hidden=True,
+              help="Format output in plain text (default)")
 @click.option('-g/-G', '--pager/--no-pager', 'pager', default=None,
               help="(Don't) view output through a pager.")
 @click.pass_obj
 def log(watson, current, from_, to, projects, tags, year, month, week, day,
-        luna, all, format_json, pager):
+        luna, all, output_format, pager):
     """
     Display each recorded session during the given timespan.
 
@@ -801,6 +910,10 @@ def log(watson, current, from_, to, projects, tags, year, month, week, day,
     `--tag` options. They can be specified several times each to add multiple
     projects or tags to the log.
 
+    You can change the output format from *plain text* to *JSON* using the
+    `--json` option or to *CSV* using the `--csv` option. Only one  of these
+    two options can be used at once.
+
     Example:
 
     \b
@@ -829,6 +942,14 @@ def log(watson, current, from_, to, projects, tags, year, month, week, day,
     Wednesday 16 April 2014 (5h 19m 18s)
             02cb269  09:53 to 12:43   2h 50m 07s  apollo11  [wheels]
             1070ddb  13:48 to 16:17   2h 29m 11s  voyager1  [antenna, sensors]
+    \b
+    $ watson log --from 2014-04-16 --to 2014-04-17 --csv
+    id,start,stop,project,tags
+    a96fcde,2014-04-17 09:15,2014-04-17 09:43,hubble,"lens, camera, transmission"
+    5e91316,2014-04-17 10:19,2014-04-17 12:59,hubble,"camera, transmission"
+    761dd51,2014-04-17 14:42,2014-04-17 15:54,voyager1,antenna
+    02cb269,2014-04-16 09:53,2014-04-16 12:43,apollo11,wheels
+    1070ddb,2014-04-16 13:48,2014-04-16 16:17,voyager1,"antenna, sensors"
     """  # noqa
     for start_time in (_ for _ in [day, week, month, luna, year, all]
                        if _ is not None):
@@ -849,18 +970,12 @@ def log(watson, current, from_, to, projects, tags, year, month, week, day,
         projects=projects or None, tags=tags or None, span=span
     )
 
-    if format_json:
-        log = [
-            {
-                'id': frame.id,
-                'start': frame.start.isoformat(),
-                'stop': frame.stop.isoformat(),
-                'project': frame.project,
-                'tags': frame.tags,
-            }
-            for frame in filtered_frames
-        ]
-        click.echo(json.dumps(log, indent=4, sort_keys=True))
+    if 'json' in output_format:
+        click.echo(frames_to_json(filtered_frames))
+        return
+
+    if 'csv' in output_format:
+        click.echo(frames_to_csv(filtered_frames))
         return
 
     frames_by_day = sorted_groupby(
@@ -994,8 +1109,12 @@ def frames(watson):
               help="Date and time of start of tracked activity")
 @click.option('-t', '--to', required=True, type=Date,
               help="Date and time of end of tracked activity")
+@click.option('-c', '--confirm-new-project', is_flag=True, default=False,
+              help="Confirm addition of new project.")
+@click.option('-b', '--confirm-new-tag', is_flag=True, default=False,
+              help="Confirm creation of new tag.")
 @click.pass_obj
-def add(watson, args, from_, to):
+def add(watson, args, from_, to, confirm_new_project, confirm_new_tag):
     """
     Add time for project with tag(s) that was not tracked live.
 
@@ -1010,9 +1129,19 @@ def add(watson, args, from_, to):
         itertools.takewhile(lambda s: not s.startswith('+'), args)
     )
 
+    # Confirm creation of new project if that option is set
+    if (watson.config.getboolean('options', 'confirm_new_project') or
+            confirm_new_project):
+        confirm_project(project, watson.projects)
+
     # Parse all the tags
     tags = parse_tags(args)
 
+    # Confirm creation of new tag(s) if that option is set
+    if (watson.config.getboolean('options', 'confirm_new_tag') or
+            confirm_new_tag):
+        confirm_tags(tags, watson.tags)
+
     # add a new frame, call watson save to update state files
     frame = watson.add(project=project, tags=tags, from_date=from_, to_date=to)
     click.echo(
@@ -1028,9 +1157,13 @@ def add(watson, args, from_, to):
 
 
 @cli.command(context_settings={'ignore_unknown_options': True})
+@click.option('-c', '--confirm-new-project', is_flag=True, default=False,
+              help="Confirm addition of new project.")
+@click.option('-b', '--confirm-new-tag', is_flag=True, default=False,
+              help="Confirm creation of new tag.")
 @click.argument('id', required=False)
 @click.pass_obj
-def edit(watson, id):
+def edit(watson, confirm_new_project, confirm_new_tag, id):
     """
     Edit a frame.
 
@@ -1090,7 +1223,15 @@ def edit(watson, id):
         try:
             data = json.loads(output)
             project = data['project']
+            # Confirm creation of new project if that option is set
+            if (watson.config.getboolean('options', 'confirm_new_project') or
+                    confirm_new_project):
+                confirm_project(project, watson.projects)
             tags = data['tags']
+            # Confirm creation of new tag(s) if that option is set
+            if (watson.config.getboolean('options', 'confirm_new_tag') or
+                    confirm_new_tag):
+                confirm_tags(tags, watson.tags)
             start = arrow.get(data['start'], datetime_format).replace(
                 tzinfo=local_tz).to('utc')
             stop = arrow.get(data['stop'], datetime_format).replace(
